@@ -4,9 +4,9 @@
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,8 @@
  */
 package ghidra;
 
-import java.io.*;
+import java.io.File;
+import java.io.IOException;
 import java.util.*;
 
 import generic.jar.ResourceFile;
@@ -35,13 +36,11 @@ public class GhidraApplicationLayout extends ApplicationLayout {
 
 	/**
 	 * Constructs a new Ghidra application layout object.
-	 * 
-	 * @throws FileNotFoundException if there was a problem getting a user
-	 *             directory.
-	 * @throws IOException if there was a problem getting the application
-	 *             properties or modules.
+	 *
+	 * @throws IOException if there was a problem getting a user directory or the application 
+	 *   properties or modules.
 	 */
-	public GhidraApplicationLayout() throws FileNotFoundException, IOException {
+	public GhidraApplicationLayout() throws IOException {
 
 		// Application root directories
 		applicationRootDirs = findGhidraApplicationRootDirs();
@@ -52,18 +51,22 @@ public class GhidraApplicationLayout extends ApplicationLayout {
 		// Application installation directory
 		applicationInstallationDir = findGhidraApplicationInstallationDir();
 
-		// Modules
-		modules = findGhidraModules();
-
 		// User directories
-		userTempDir = ApplicationUtilities.getDefaultUserTempDir(getApplicationProperties());
+		userTempDir = ApplicationUtilities
+				.getDefaultUserTempDir(getApplicationProperties().getApplicationName());
 		userCacheDir = ApplicationUtilities.getDefaultUserCacheDir(getApplicationProperties());
 		userSettingsDir = ApplicationUtilities.getDefaultUserSettingsDir(getApplicationProperties(),
 			getApplicationInstallationDir());
 
 		// Extensions
-		extensionInstallationDir = findExtensionInstallationDirectory();
+		extensionInstallationDirs = findExtensionInstallationDirectories();
 		extensionArchiveDir = findExtensionArchiveDirectory();
+
+		// Patch directory
+		patchDir = findPatchDirectory();
+
+		// Modules
+		modules = findGhidraModules();
 	}
 
 	/**
@@ -72,15 +75,12 @@ public class GhidraApplicationLayout extends ApplicationLayout {
 	 * <p>
 	 * This is used when something external to Ghidra needs Ghidra's layout
 	 * (like the Eclipse GhidraDevPlugin).
-	 * 
+	 *
 	 * @param applicationInstallationDir The application installation directory.
-	 * @throws FileNotFoundException if there was a problem getting a user
-	 *             directory.
-	 * @throws IOException if there was a problem getting the application
-	 *             properties.
+	 * @throws IOException if there was a problem getting a user directory or the application
+	 *   properties.
 	 */
-	public GhidraApplicationLayout(File applicationInstallationDir)
-			throws FileNotFoundException, IOException {
+	public GhidraApplicationLayout(File applicationInstallationDir) throws IOException {
 
 		// Application installation directory
 		this.applicationInstallationDir = new ResourceFile(applicationInstallationDir);
@@ -92,29 +92,37 @@ public class GhidraApplicationLayout extends ApplicationLayout {
 		// Application properties
 		applicationProperties = new ApplicationProperties(applicationRootDirs);
 
-		// Modules
-		modules = findGhidraModules();
-
 		// User directories
-		userTempDir = ApplicationUtilities.getDefaultUserTempDir(getApplicationProperties());
+		userTempDir = ApplicationUtilities
+				.getDefaultUserTempDir(getApplicationProperties().getApplicationName());
 		userCacheDir = ApplicationUtilities.getDefaultUserCacheDir(getApplicationProperties());
 		userSettingsDir = ApplicationUtilities.getDefaultUserSettingsDir(getApplicationProperties(),
 			getApplicationInstallationDir());
+
+		// Extensions
+		extensionInstallationDirs = findExtensionInstallationDirectories();
+		extensionArchiveDir = findExtensionArchiveDirectory();
+
+		// Patch directory
+		patchDir = findPatchDirectory();
+
+		// Modules
+		modules = findGhidraModules();
 	}
 
 	/**
 	 * Finds the application root directories for this application layout.
-	 * 
-	 * @return A collection of the application root directories for this layout.
+	 *
+	 * @return A {@link SequencedCollection} of the application root directories for this layout.
 	 */
-	protected Collection<ResourceFile> findGhidraApplicationRootDirs() {
+	protected SequencedCollection<ResourceFile> findGhidraApplicationRootDirs() {
 		return ApplicationUtilities.findDefaultApplicationRootDirs();
 	}
 
 	/**
 	 * Finds the application installation directory for this Ghidra application
 	 * layout.
-	 * 
+	 *
 	 * @return The application installation directory for this Ghidra
 	 *         application layout. Could be null if there is no application
 	 *         installation directory.
@@ -125,7 +133,7 @@ public class GhidraApplicationLayout extends ApplicationLayout {
 		}
 
 		ResourceFile dir = applicationRootDirs.iterator().next().getParentFile();
-		if (SystemUtilities.isInDevelopmentMode()) {
+		if (SystemUtilities.isInDevelopmentMode() && applicationRootDirs.size() > 1) {
 			dir = dir.getParentFile();
 		}
 		return dir;
@@ -133,7 +141,7 @@ public class GhidraApplicationLayout extends ApplicationLayout {
 
 	/**
 	 * Finds the modules for this Ghidra application layout.
-	 * 
+	 *
 	 * @return The modules for this Ghidra application layout.
 	 * @throws IOException if there was a problem finding the modules on disk.
 	 */
@@ -141,45 +149,66 @@ public class GhidraApplicationLayout extends ApplicationLayout {
 
 		// Find standard module root directories from within the application root directories
 		Collection<ResourceFile> moduleRootDirectories =
-			ModuleUtilities.findModuleRootDirectories(applicationRootDirs, new ArrayList<>());
-		
-		// Examine the classpath to look for modules outside of the application root directories.
-		// These might exist if Ghidra was launched from an Eclipse project that resides
-		// external to the Ghidra installation.
-		for (String entry : System.getProperty("java.class.path", "").split(File.pathSeparator)) {
-			final ResourceFile classpathEntry = new ResourceFile(entry);
+			ModuleUtilities.findModuleRootDirectories(applicationRootDirs, new LinkedHashSet<>());
 
-			// We only care about directories (skip jars)
-			if (!classpathEntry.isDirectory()) {
-				continue;
-			}
+		// Find installed extension modules
+		for (ResourceFile extensionInstallDir : extensionInstallationDirs) {
 
-			// Skip classpath entries that live in an application root directory...we've already
-			// found those.
-			if (applicationRootDirs.stream().anyMatch(dir -> FileUtilities.isPathContainedWithin(
-				dir.getFile(false), classpathEntry.getFile(false)))) {
-				continue;
-			}
+			FileUtilities.forEachFile(extensionInstallDir, extensionDir -> {
+				// Skip extensions in an application root directory... already found those.
+				if (FileUtilities.isPathContainedWithin(applicationRootDirs, extensionDir)) {
+					return;
+				}
 
-			// We are going to assume that the classpath entry is in a subdirectory of the module
-			// directory (i.e., bin/), so only check parent directory for the module.
-			ResourceFile classpathEntryParent = classpathEntry.getParentFile();
-			if (classpathEntryParent != null &&
-				ModuleUtilities.isModuleDirectory(classpathEntryParent)) {
-				moduleRootDirectories.add(classpathEntryParent);
+				// Skip extensions slated for cleanup
+				if (ModuleUtilities.isUninstalled(extensionDir)) {
+					return;
+				}
+
+				moduleRootDirectories.add(extensionDir);
+			});
+		}
+
+		// Add external modules defined via a system property. This will typically be used by
+		// user's developing 3rd party modules from something like Eclipse.
+		String externalModules = System.getProperty("ghidra.external.modules", "");
+		if (!externalModules.isBlank()) {
+			for (String path : externalModules.split(File.pathSeparator)) {
+				ResourceFile eclipseProjectDir = new ResourceFile(path);
+				if (ModuleUtilities.isModuleDirectory(eclipseProjectDir)) {
+					moduleRootDirectories.add(eclipseProjectDir);
+				}
 			}
 		}
 
 		return ModuleUtilities.findModules(applicationRootDirs, moduleRootDirectories);
 	}
 
+	/**
+	 * Returns the directory that allows users to add jar and class files to override existing
+	 * distribution files
+	 * @return the patch dir; null if not in a distribution
+	 */
+	protected ResourceFile findPatchDirectory() {
+
+		if (SystemUtilities.isInDevelopmentMode()) {
+			return null;
+		}
+
+		if (applicationInstallationDir == null) {
+			return null;
+		}
+
+		return new ResourceFile(applicationInstallationDir, "Ghidra/patch");
+	}
+
 	/**
 	 * Returns the directory where all Ghidra extension archives are stored.
 	 * This should be at the following location:<br>
 	 * <ul>
-	 * <li><code>[application root]/Extensions/Ghidra</code></li>
+	 * <li><code>{install dir}/Extensions/Ghidra</code></li>
 	 * </ul>
-	 * 
+	 *
 	 * @return the archive folder, or null if can't be determined
 	 */
 	protected ResourceFile findExtensionArchiveDirectory() {
@@ -195,33 +224,32 @@ public class GhidraApplicationLayout extends ApplicationLayout {
 	}
 
 	/**
-	 * Returns the directory where all Ghidra extension archives should be
-	 * installed. This should be at the following location:<br>
+	 * Returns a prioritized list of directories where Ghidra extensions are installed. These
+	 * should be at the following locations:<br>
 	 * <ul>
-	 * <li><code>[application install dir]/Ghidra/Extensions</code></li>
-	 * <li><code>ghidra/Ghidra/Extensions</code> (development mode)</li>
+	 * <li>{@code [user settings dir]/Extensions}</li>
+	 * <li>{@code [application root dirs]/Extensions}</li>
 	 * </ul>
-	 * 
+	 *
 	 * @return the install folder, or null if can't be determined
 	 */
-	protected ResourceFile findExtensionInstallationDirectory() {
+	protected List<ResourceFile> findExtensionInstallationDirectories() {
+
+		List<ResourceFile> dirs = new ArrayList<>();
+		dirs.add(new ResourceFile(new File(userSettingsDir, "Extensions")));
 
-		// Would like to find a better way to do this, but for the moment this seems the
-		// only solution. We want to get the 'Extensions' directory in ghidra, but there's 
-		// no way to retrieve that directory directly. We can only get the full set of 
-		// application root dirs and search for it, hoping we don't encounter one with the
-		// name 'Extensions' in one of the other root dirs.
 		if (SystemUtilities.isInDevelopmentMode()) {
-			ResourceFile rootDir = getApplicationRootDirs().iterator().next();
-			File temp = new File(rootDir.getFile(false), "Extensions");
-			if (temp.exists()) {
-				return new ResourceFile(temp);
+			for (ResourceFile rootDir : getApplicationRootDirs()) {
+				File temp = new File(rootDir.getFile(false), "Extensions");
+				if (temp.exists()) {
+					dirs.add(new ResourceFile(temp)); // i.e., ghidra/Ghidra/Extensions
+				}
 			}
-
-			return null;
+		}
+		else {
+			dirs.add(new ResourceFile(applicationInstallationDir, "Ghidra/Extensions"));
 		}
 
-		ResourceFile installDir = findGhidraApplicationInstallationDir();
-		return new ResourceFile(installDir, "Ghidra/Extensions");
+		return dirs;
 	}
 }
