@@ -20,10 +20,8 @@ import java.util.*;
 
 import ghidra.program.database.register.InMemoryRangeMapAdapter;
 import ghidra.program.model.address.*;
-import ghidra.program.model.lang.Register;
-import ghidra.program.model.lang.RegisterValue;
+import ghidra.program.model.lang.*;
 import ghidra.program.model.listing.ContextChangeException;
-import ghidra.util.exception.AssertException;
 import ghidra.util.exception.CancelledException;
 import ghidra.util.task.TaskMonitor;
 
@@ -34,8 +32,8 @@ abstract public class AbstractStoredProgramContext extends AbstractProgramContex
 
 	private Set<Register> registersWithValues; // cached set, recomputed whenever null
 
-	protected AbstractStoredProgramContext(Register[] registers) {
-		super(registers);
+	protected AbstractStoredProgramContext(Language language) {
+		super(language);
 		registerValueMap = new HashMap<>();
 		defaultRegisterValueMap = new HashMap<>();
 	}
@@ -144,7 +142,7 @@ abstract public class AbstractStoredProgramContext extends AbstractProgramContex
 		}
 
 		RegisterValueStore store = map.get(register.getBaseRegister());
-		if (store == null) {
+		if (store == null || store.isEmpty()) {
 			return null;
 		}
 
@@ -154,8 +152,8 @@ abstract public class AbstractStoredProgramContext extends AbstractProgramContex
 	@Override
 	public AddressRangeIterator getRegisterValueAddressRanges(Register register) {
 		RegisterValueStore store = registerValueMap.get(register.getBaseRegister());
-		if (store == null) {
-			return new AddressSet().getAddressRanges();
+		if (store == null || store.isEmpty()) {
+			return new EmptyAddressRangeIterator();
 		}
 		return new RegisterAddressRangeIterator(register, store.getAddressRangeIterator(),
 			registerValueMap);
@@ -164,7 +162,7 @@ abstract public class AbstractStoredProgramContext extends AbstractProgramContex
 	@Override
 	public AddressRange getRegisterValueRangeContaining(Register register, Address addr) {
 		RegisterValueStore store = registerValueMap.get(register.getBaseRegister());
-		if (store == null) {
+		if (store == null || store.isEmpty()) {
 			return new AddressRangeImpl(addr, addr);
 		}
 		return store.getValueRangeContaining(addr);
@@ -174,8 +172,8 @@ abstract public class AbstractStoredProgramContext extends AbstractProgramContex
 	public AddressRangeIterator getRegisterValueAddressRanges(Register register, Address start,
 			Address end) {
 		RegisterValueStore store = registerValueMap.get(register.getBaseRegister());
-		if (store == null) {
-			return new AddressSet().getAddressRanges();
+		if (store == null || store.isEmpty()) {
+			return new EmptyAddressRangeIterator();
 		}
 		return new RegisterAddressRangeIterator(register, store.getAddressRangeIterator(start, end),
 			registerValueMap);
@@ -184,8 +182,8 @@ abstract public class AbstractStoredProgramContext extends AbstractProgramContex
 	@Override
 	public AddressRangeIterator getDefaultRegisterValueAddressRanges(Register register) {
 		RegisterValueStore store = defaultRegisterValueMap.get(register.getBaseRegister());
-		if (store == null) {
-			return new AddressSet().getAddressRanges();
+		if (store == null || store.isEmpty()) {
+			return new EmptyAddressRangeIterator();
 		}
 		return new RegisterAddressRangeIterator(register, store.getAddressRangeIterator(),
 			defaultRegisterValueMap);
@@ -195,8 +193,8 @@ abstract public class AbstractStoredProgramContext extends AbstractProgramContex
 	public AddressRangeIterator getDefaultRegisterValueAddressRanges(Register register,
 			Address start, Address end) {
 		RegisterValueStore store = defaultRegisterValueMap.get(register.getBaseRegister());
-		if (store == null) {
-			return new AddressSet().getAddressRanges();
+		if (store == null || store.isEmpty()) {
+			return new EmptyAddressRangeIterator();
 		}
 		return new RegisterAddressRangeIterator(register, store.getAddressRangeIterator(start, end),
 			defaultRegisterValueMap);
@@ -206,7 +204,7 @@ abstract public class AbstractStoredProgramContext extends AbstractProgramContex
 	public Register[] getRegistersWithValues() {
 		if (registersWithValues == null) {
 			registersWithValues = new HashSet<>();
-			for (Register register : registers) {
+			for (Register register : language.getRegisters()) {
 				RegisterValueStore store = registerValueMap.get(register.getBaseRegister());
 				if (store != null && !store.isEmpty()) {
 					registersWithValues.add(register);
@@ -249,8 +247,9 @@ abstract public class AbstractStoredProgramContext extends AbstractProgramContex
 	@Override
 	public void remove(Address start, Address end, Register register)
 			throws ContextChangeException {
-		if (start.getAddressSpace() != end.getAddressSpace()) {
-			throw new AssertException("start and end address must be in the same address space");
+		if (!start.getAddressSpace().equals(end.getAddressSpace())) {
+			throw new IllegalArgumentException(
+				"start and end address must be within the same address space");
 		}
 		RegisterValueStore values = registerValueMap.get(register.getBaseRegister());
 		if (values != null) {
@@ -260,8 +259,9 @@ abstract public class AbstractStoredProgramContext extends AbstractProgramContex
 	}
 
 //	public void removeDefault(Address start, Address end, Register register) {
-//		if (start.getAddressSpace() != end.getAddressSpace()) {
-//			throw new AssertException("start and end address must be in the same address space");
+//		if (!start.getAddressSpace().equals(end.getAddressSpace())) {
+//			throw new IllegalArgumentException(
+//				"start and end address must be within the same address space");
 //		}
 //		invalidateCache();
 //		RegisterValueStore values = defaultRegisterValueMap.get(register.getBaseRegister());
@@ -273,8 +273,9 @@ abstract public class AbstractStoredProgramContext extends AbstractProgramContex
 	@Override
 	public void setValue(Register register, Address start, Address end, BigInteger value)
 			throws ContextChangeException {
-		if (start.getAddressSpace() != end.getAddressSpace()) {
-			throw new AssertException("start and end address must be in the same address space");
+		if (!start.getAddressSpace().equals(end.getAddressSpace())) {
+			throw new IllegalArgumentException(
+				"start and end address must be within the same address space");
 		}
 		if (value == null) {
 			remove(start, end, register);
@@ -285,8 +286,9 @@ abstract public class AbstractStoredProgramContext extends AbstractProgramContex
 
 	@Override
 	public void setDefaultValue(RegisterValue registerValue, Address start, Address end) {
-		if (start.getAddressSpace() != end.getAddressSpace()) {
-			throw new AssertException("start and end address must be in the same address space");
+		if (!start.getAddressSpace().equals(end.getAddressSpace())) {
+			throw new IllegalArgumentException(
+				"start and end address must be within the same address space");
 		}
 		Register baseRegister = registerValue.getRegister().getBaseRegister();
 		RegisterValueStore store = defaultRegisterValueMap.get(baseRegister);
@@ -307,7 +309,7 @@ abstract public class AbstractStoredProgramContext extends AbstractProgramContex
 		if (space.isOverlaySpace()) {
 			address = ((OverlayAddressSpace) space).translateAddress(address, true);
 		}
-		// there is a wierd deleted overlay case that the following code handles - sortof
+		// there is a weird deleted overlay case that the following code handles - sortof
 		else if (space.getType() == AddressSpace.TYPE_UNKNOWN) {
 			return new RegisterValue(register);
 		}
