@@ -28,11 +28,11 @@ public enum OpenBSDIoctls implements Ioctls {
 
 	@Override
 	public long TIOCSCTTY() {
-		return 0x20007461;
+		return 0x20007461L;
 	}
 
 	@Override
 	public long TIOCSWINSZ() {
-		return 0x80087467;
+		return 0x80087467L;
 	}
 }
